@@ -37,30 +37,6 @@ class CControllerAuditLogList extends CController {
 
 		$ret = $this->validateInput($fields);
 
-		if ($ret) {
-			$fields = [];
-
-			if ($this->getInput('filter_resourceid', '') !== '') {
-				$fields['filter_resourceid'] = 'id';
-			}
-
-			if ($this->getInput('filter_recordsetid', '') !== '') {
-				$fields['filter_recordsetid'] = 'cuid';
-			}
-
-			if ($fields) {
-				$validator = new CNewValidator($this->getInputAll(), $fields);
-
-				foreach ($validator->getAllErrors() as $error) {
-					info($error);
-				}
-
-				if ($validator->isErrorFatal() || $validator->isError()) {
-					$ret = false;
-				}
-			}
-		}
-
 		if (!$ret) {
 			$this->setResponse(new CControllerResponseFatal());
 		}
@@ -217,7 +193,8 @@ class CControllerAuditLogList extends CController {
 			CAudit::ACTION_UPDATE => _('Update'),
 			CAudit::ACTION_DELETE => _('Delete'),
 			CAudit::ACTION_EXECUTE => _('Execute'),
-			CAudit::ACTION_HISTORY_CLEAR => _('History clear')
+			CAudit::ACTION_HISTORY_CLEAR => _('History clear'),
+			CAudit::ACTION_CONFIG_REFRESH => _('Configuration refresh')
 		];
 	}
 
@@ -263,7 +240,9 @@ class CControllerAuditLogList extends CController {
 			CAudit::RESOURCE_TEMPLATE_DASHBOARD => _('Template dashboard'),
 			CAudit::RESOURCE_TRIGGER => _('Trigger'),
 			CAudit::RESOURCE_TRIGGER_PROTOTYPE => _('Trigger prototype'),
+			CAudit::RESOURCE_TEMPLATE_GROUP => _('Template group'),
 			CAudit::RESOURCE_USER => _('User'),
+			CAudit::RESOURCE_USERDIRECTORY => _('User directory'),
 			CAudit::RESOURCE_USER_GROUP => _('User group'),
 			CAudit::RESOURCE_USER_ROLE => _('User role'),
 			CAudit::RESOURCE_VALUE_MAP => _('Value map')
