@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@ class CControllerFavouriteCreate extends CController {
 
 	protected function checkInput() {
 		$fields = [
-			'object' =>		'fatal|required|in graphid,itemid,screenid,slideshowid,sysmapid',
+			'object' =>		'fatal|required|in itemid,sysmapid',
 			'objectid' =>	'fatal|required|id'
 		];
 
@@ -42,10 +42,7 @@ class CControllerFavouriteCreate extends CController {
 
 	protected function doAction() {
 		$profile = [
-			'graphid' => 'web.favorite.graphids',
 			'itemid' => 'web.favorite.graphids',
-			'screenid' => 'web.favorite.screenids',
-			'slideshowid' => 'web.favorite.screenids',
 			'sysmapid' => 'web.favorite.sysmapids'
 		];
 
@@ -59,9 +56,16 @@ class CControllerFavouriteCreate extends CController {
 		$result = DBend($result);
 
 		if ($result) {
-			$data['main_block'] = 'document.getElementById("addrm_fav").title = "'._('Remove from favourites').'";'."\n".
-				'document.getElementById("addrm_fav").onclick = function() { rm4favorites("'.$object.'", "'.$objectid.'"); }'."\n".
-				'switchElementClass("addrm_fav", "btn-add-fav", "btn-remove-fav");';
+			$data['main_block'] = '
+				var addrm_fav = document.getElementById("addrm_fav");
+
+				if (addrm_fav !== null) {
+					addrm_fav.title = "'._('Remove from favorites').'";
+					addrm_fav.onclick = () => rm4favorites("'.$object.'", "'.$objectid.'");
+					addrm_fav.classList.add("btn-remove-fav");
+					addrm_fav.classList.remove("btn-add-fav");
+				}
+			';
 		}
 		else {
 			$data['main_block'] = '';
