@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -28,9 +28,7 @@ $page['title'] = _('Details of web scenario');
 $page['file'] = 'httpdetails.php';
 $page['scripts'] = ['class.calendar.js', 'gtlc.js', 'flickerfreescreen.js', 'layout.mode.js'];
 $page['type'] = detect_page_type(PAGE_TYPE_HTML);
-
-CView::$has_web_layout_mode = true;
-$page['web_layout_mode'] = CView::getLayoutMode();
+$page['web_layout_mode'] = CViewHelper::loadLayoutMode();
 
 require_once dirname(__FILE__).'/include/page_header.php';
 
@@ -74,28 +72,12 @@ $timeline = getTimeSelectorPeriod($timeselector_options);
 
 $http_test_name = CMacrosResolverHelper::resolveHttpTestName($httptest['hostid'], $httptest['name']);
 
-// Create details widget.
 $details_screen = CScreenBuilder::getScreen([
 	'resourcetype' => SCREEN_RESOURCE_HTTPTEST_DETAILS,
 	'mode' => SCREEN_MODE_JS,
 	'dataId' => 'httptest_details'
 ] + $timeline);
 
-(new CWidget())
-	->setTitle(_('Details of web scenario').': '.$http_test_name)
-	->setWebLayoutMode($page['web_layout_mode'])
-	->setControls((new CTag('nav', true,
-		(new CForm())
-			->cleanItems()
-			->addItem((new CList())->addItem(get_icon('fullscreen')))
-		))
-			->setAttribute('aria-label', _('Content controls'))
-	)
-	->addItem($details_screen->get())
-	->show();
-
-echo BR();
-
 $graphs = [];
 
 // dims
@@ -200,9 +182,19 @@ CScreenBuilder::insertScreenStandardJs($graph_in->timeline);
 
 // Create graphs widget.
 $widget = (new CWidget())
+	->setTitle(_('Details of web scenario').': '.$http_test_name)
 	->setWebLayoutMode($page['web_layout_mode'])
+	->setControls((new CTag('nav', true,
+		(new CForm())
+			->cleanItems()
+			->addItem((new CList())->addItem(get_icon('kioskmode', ['mode' => $page['web_layout_mode']])))
+		))
+			->setAttribute('aria-label', _('Content controls'))
+	)
+	->addItem($details_screen->get())
+	->addItem(new CTag('br'))
 	->addItem(
-		(new CFilter(new CUrl()))
+		(new CFilter())
 			->setProfile($timeline['profileIdx'], $timeline['profileIdx2'])
 			->setActiveTab(CProfile::get($timeline['profileIdx'].'.active', 1))
 			->addTimeSelector($timeline['from'], $timeline['to'], $page['web_layout_mode'] != ZBX_LAYOUT_KIOSKMODE)
