@@ -237,7 +237,7 @@ class DB {
 	}
 
 	/**
-	 * Returns true if the table $tableName has the $fieldName field.
+	 * Returns true if the table $tableName has the $fieldName field defined in its schema.
 	 *
 	 * @static
 	 *
@@ -252,6 +252,20 @@ class DB {
 		return isset($schema['fields'][$fieldName]);
 	}
 
+	/**
+	 * Check that a field is present in a database table.
+	 *
+	 * @static
+	 *
+	 * @param string $table_name
+	 * @param string $field_name
+	 *
+	 * @return bool
+	 */
+	public static function dbFieldExists(string $table_name, string $field_name) {
+		return self::getDbBackend()->dbFieldExists($table_name, $field_name);
+	}
+
 	/**
 	 * Returns length of the field.
 	 *
@@ -445,7 +459,7 @@ class DB {
 								break;
 
 							case ZBX_DB_POSTGRESQL:
-								$values[$field] = "'".pg_escape_bytea($values[$field])."'";
+								$values[$field] = "'".pg_escape_bytea($DB['DB'], $values[$field])."'";
 								break;
 
 							case ZBX_DB_ORACLE:
@@ -1075,6 +1089,27 @@ class DB {
 		return ($table_alias !== null) ? $table_alias.'.'.$field_name : $field_name;
 	}
 
+	/**
+	 * Convert field to uppercase or substitute it with its pre-upcased variant.
+	 *
+	 * @param string      $field_name
+	 * @param string      $table_name
+	 * @param string|null $table_alias
+	 *
+	 * @return string
+	 */
+	public static function uppercaseField(string $field_name, string $table_name, string $table_alias = null): string {
+		if ($table_alias === null) {
+			$table_alias = $table_name;
+		}
+
+		if ($field_name === 'name' && self::dbFieldExists($table_name, 'name_upper')) {
+			return $table_alias.'.name_upper';
+		}
+
+		return 'UPPER('.$table_alias.'.'.$field_name.')';
+	}
+
 	/**
 	 * Builds an SQL parts array from the given options.
 	 *
@@ -1253,7 +1288,7 @@ class DB {
 					$pattern = zbx_dbstr($pattern);
 				}
 
-				$search[] = 'UPPER('.self::fieldId($field_name, $table_alias).') LIKE '.$pattern." ESCAPE '!'";
+				$search[] = self::uppercaseField($field_name, $table_name, $table_alias).' LIKE '.$pattern." ESCAPE '!'";
 			}
 		}
 
