@@ -1,7 +1,7 @@
-<?php declare(strict_types = 1);
+<?php declare(strict_types = 0);
 /*
 ** Zabbix
-** Copyright (C) 2001-2021 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -78,6 +78,14 @@ class CSettingsHelper extends CConfigGeneralHelper {
 	public const SESSION_KEY = 'session_key';
 	public const URL = 'url';
 	public const SCHEDULED_REPORT_TEST_TIMEOUT = 'report_test_timeout';
+	public const DBVERSION_STATUS = 'dbversion_status';
+	public const AUDITLOG_ENABLED = 'auditlog_enabled';
+	public const GEOMAPS_TILE_PROVIDER = 'geomaps_tile_provider';
+	public const GEOMAPS_TILE_URL = 'geomaps_tile_url';
+	public const GEOMAPS_MAX_ZOOM = 'geomaps_max_zoom';
+	public const GEOMAPS_ATTRIBUTION = 'geomaps_attribution';
+	public const HA_FAILOVER_DELAY = 'ha_failover_delay';
+	public const VAULT_PROVIDER = 'vault_provider';
 
 	/**
 	 * Settings API object parameters array.
@@ -98,7 +106,8 @@ class CSettingsHelper extends CConfigGeneralHelper {
 				: API::Settings()->get(['output' => 'extend']);
 		}
 		else if (!$is_global && $param && !array_key_exists($param, self::$params)) {
-			self::$params = API::Settings()->get(['output' => 'extend']) + self::$params;
+			$settings = API::Settings()->get(['output' => 'extend']);
+			self::$params = is_array($settings) ? ($settings + self::$params) : false;
 		}
 
 		if (self::$params === false) {
@@ -120,4 +129,17 @@ class CSettingsHelper extends CConfigGeneralHelper {
 
 		return array_key_exists($name, self::$params) ? self::$params[$name] : null;
 	}
+
+	/**
+	 * @return array
+	 */
+	public static function getDbVersionStatus(): array {
+		$dbversion_status = json_decode((string) self::getGlobal(self::DBVERSION_STATUS), true);
+
+		if (!is_array($dbversion_status)) {
+			return [];
+		}
+
+		return $dbversion_status;
+	}
 }
