@@ -55,16 +55,19 @@ class CActionButtonList extends CObject {
 	protected $selected_count_element = null;
 
 	/**
-	 * @param string       $action_name                 Name of submit buttons.
-	 * @param string       $checkboxes_name             Name of paramerer into which checked checkboxes will be put in.
-	 * @param array        $buttons_data                Buttons data array.
-	 * @param string       $buttons_data[]['name']      Button caption.
-	 * @param string       $buttons_data[]['confirm']   Confirmation text (optional).
-	 * @param string       $buttons_data[]['redirect']  Redirect URL (optional).
-	 * @param bool         $buttons_data[]['disabled']  Set button state disabled (optional).
-	 * @param CTag         $buttons_data[]['content']   A HTML tag. For example a CButton wrapped in CList object.
-	 * @param string|null  $name_prefix                 Prefix for sessionStorage used for storing currently selected
-	 *                                                  checkboxes.
+	 * @param string       $action_name                   Name of submit buttons.
+	 * @param string       $checkboxes_name               Name of paramerer into which checked checkboxes will be put
+	 *                                                    in.
+	 * @param array        $buttons_data                  Buttons data array.
+	 * @param string       $buttons_data[]['name']        Button caption.
+	 * @param string       $buttons_data[]['confirm']     Confirmation text (optional).
+	 * @param string       $buttons_data[]['redirect']    Redirect URL (optional).
+	 * @param bool         $buttons_data[]['disabled']    Set button state disabled (optional).
+	 * @param array        $buttons_data[]['attributes']  Set additional HTML attributes where array key is attribute
+	 *                                                    name array value is the attribute value.
+	 * @param CTag         $buttons_data[]['content']     A HTML tag. For example a CButton wrapped in CList object.
+	 * @param string|null  $name_prefix                   Prefix for sessionStorage used for storing currently selected
+	 *                                                    checkboxes.
 	 */
 	function __construct($action_name, $checkboxes_name, array $buttons_data, $name_prefix = null) {
 		$this->checkboxes_name = $checkboxes_name;
@@ -79,17 +82,25 @@ class CActionButtonList extends CObject {
 					->addClass(ZBX_STYLE_BTN_ALT)
 					->removeAttribute('id');
 
+				if (array_key_exists('attributes', $button_data) && is_array($button_data['attributes'])
+						&& $button_data['attributes']) {
+					foreach ($button_data['attributes'] as $attr_name => $attr_value) {
+						$button->setAttribute($attr_name, $attr_value);
+					}
+				}
+
 				if (array_key_exists('redirect', $button_data)) {
 					$button
 						// Removing parameters not to conflict with the redirecting URL.
 						->removeAttribute('name')
 						->removeAttribute('value')
+						->setAttribute('data-redirect', $button_data['redirect'])
 						->onClick('var $_form = jQuery(this).closest("form");'.
 							// Save the original form action.
 							'if (!$_form.data("action")) {'.
 								'$_form.data("action", $_form.attr("action"));'.
 							'}'.
-							'$_form.attr("action", '.json_encode($button_data['redirect']).');'
+							'$_form.attr("action", this.dataset.redirect);'
 						);
 				}
 				else {
