@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -30,35 +30,41 @@ class CWidgetForm {
 	 */
 	protected $data;
 
-	public function __construct($data, $type) {
-		$this->data = CJs::decodeJson($data);
+	protected $templateid;
+
+	public function __construct($data, $templateid, $type) {
+		$this->data = json_decode($data, true);
+
+		$this->templateid = $templateid;
 
 		$this->fields = [];
 
-		// Refresh interval field.
-		$default_rf_rate = '';
+		if ($templateid === null) {
+			// Refresh interval field.
+			$default_rf_rate = '';
 
-		foreach (CWidgetConfig::getRfRates() as $rf_rate => $label) {
-			if ($rf_rate == CWidgetConfig::getDefaultRfRate($type)) {
-				$default_rf_rate = $label;
-				break;
+			foreach (CWidgetConfig::getRfRates() as $rf_rate => $label) {
+				if ($rf_rate == CWidgetConfig::getDefaultRfRate($type)) {
+					$default_rf_rate = $label;
+					break;
+				}
 			}
+
+			$rf_rates = [
+				-1 => _('Default').' ('.$default_rf_rate.')'
+			];
+			$rf_rates += CWidgetConfig::getRfRates();
+
+			$rf_rate_field = (new CWidgetFieldSelect('rf_rate', _('Refresh interval'), $rf_rates))
+				->setDefault(-1);
+
+			if (array_key_exists('rf_rate', $this->data)) {
+				$rf_rate_field->setValue($this->data['rf_rate']);
+			}
+
+			$this->fields[$rf_rate_field->getName()] = $rf_rate_field;
 		}
 
-		$rf_rates = [
-			-1 => _('Default').' ('.$default_rf_rate.')'
-		];
-		$rf_rates += CWidgetConfig::getRfRates();
-
-		$rf_rate_field = (new CWidgetFieldComboBox('rf_rate', _('Refresh interval'), $rf_rates))
-			->setDefault(-1);
-
-		if (array_key_exists('rf_rate', $this->data)) {
-			$rf_rate_field->setValue($this->data['rf_rate']);
-		}
-
-		$this->fields[$rf_rate_field->getName()] = $rf_rate_field;
-
 		// Add Columns and Rows fields for Iterator widgets.
 
 		if (CWidgetConfig::isIterator($type)) {
@@ -114,16 +120,31 @@ class CWidgetForm {
 	 * @return array
 	 */
 	protected static function convertDottedKeys(array $data) {
+		// API doesn't guarantee fields to be retrieved in same order as stored. Sorting by key...
+		uksort($data, function ($key1, $key2) {
+			foreach (['key1', 'key2'] as $var) {
+				if (preg_match('/^([a-z]+)\.([a-z_]+)\.(\d+)\.(\d+)$/', (string) $$var, $matches) === 1) {
+					$$var = $matches[1].'.'.$matches[3].'.'.$matches[2].'.'.$matches[4];
+				}
+				elseif (preg_match('/^([a-z]+)\.([a-z_]+)\.(\d+)$/', (string) $$var, $matches) === 1) {
+					$$var = $matches[1].'.'.$matches[3].'.'.$matches[2];
+				}
+			}
+
+			return strnatcmp((string) $key1, (string) $key2);
+		});
+
+		// Converting dot-delimited keys to the arrays.
 		foreach ($data as $key => $value) {
-			if (preg_match('/^([a-z]+)\.([a-z_]+)\.(\d+)\.(\d+)$/', $key, $matches) === 1) {
+			if (preg_match('/^([a-z]+)\.([a-z_]+)\.(\d+)\.(\d+)$/', (string) $key, $matches) === 1) {
 				$data[$matches[1]][$matches[3]][$matches[2]][$matches[4]] = $value;
 				unset($data[$key]);
 			}
-			elseif (preg_match('/^([a-z]+)\.([a-z_]+)\.(\d+)$/', $key, $matches) === 1) {
+			elseif (preg_match('/^([a-z]+)\.([a-z_]+)\.(\d+)$/', (string) $key, $matches) === 1) {
 				$data[$matches[1]][$matches[3]][$matches[2]] = $value;
 				unset($data[$key]);
 			}
-			elseif (preg_match('/^([a-z]+)\.(\d+)$/', $key, $matches) === 1) {
+			elseif (preg_match('/^([a-z]+)\.(\d+)$/', (string) $key, $matches) === 1) {
 				$data[$matches[1]][$matches[2]] = $value;
 				unset($data[$key]);
 			}
@@ -163,7 +184,7 @@ class CWidgetForm {
 	 * @param bool $strict  Enables more strict validation of the form fields.
 	 *                      Must be enabled for validation of input parameters in the widget configuration form.
 	 *
-	 * @return bool
+	 * @return array
 	 */
 	public function validate($strict = false) {
 		$errors = [];
