@@ -26,7 +26,7 @@
  *  - show_problems - show problems in hintbox when mouse is moved over the problem zone;
  *  - min_period - min period in seconds that must be s-boxed to change the data in dashboard timeselector.
  */
-jQuery(function ($) {
+jQuery(function() {
 	"use strict";
 
 	// Makes SBox selection cancelable pressing Esc.
@@ -47,12 +47,12 @@ jQuery(function ($) {
 			data = graph.data('options');
 
 		if (data) {
-			if (!data.isHintBoxFrozen) {
+			if (!data.isHintBoxFrozen && !data.isTriggerHintBoxFrozen) {
 				graph.data('widget')._resumeUpdating();
 			}
 
-			$('.svg-graph-selection', graph).attr({'width': 0, 'height': 0});
-			$('.svg-graph-selection-text', graph).text('');
+			jQuery('.svg-graph-selection', graph).attr({'width': 0, 'height': 0});
+			jQuery('.svg-graph-selection-text', graph).text('');
 			graph.data('options').boxing = false;
 		}
 
@@ -76,7 +76,7 @@ jQuery(function ($) {
 		});
 
 		if (widgets_boxing == 0 || (e && 'keyCode' in e && e.keyCode == 27)) {
-			$(document)
+			jQuery(document)
 				.off('selectstart', disableSelect)
 				.off('keydown', sBoxKeyboardInteraction)
 				.off('mousemove', moveSBoxMouse)
@@ -90,7 +90,7 @@ jQuery(function ($) {
 		var data = graph.data('options'),
 			hbox = graph.data('hintbox') || null;
 
-		if (hbox !== null && data.isHintBoxFrozen === false) {
+		if (hbox !== null && (data.isHintBoxFrozen === false && data.isTriggerHintBoxFrozen === false)) {
 			graph.removeAttr('data-expanded');
 			removeFromOverlaysStack(graph.hintboxid);
 			graph.off('mouseup', makeHintboxStatic);
@@ -118,7 +118,14 @@ jQuery(function ($) {
 		if (content) {
 			// Should be put inside hintBoxItem to use functionality of hintBox.
 			graph.hintBoxItem = hintBox.createBox(e, graph, content, '', true, 'top: 0; left: 0', graph.parent());
-			data.isHintBoxFrozen = true;
+
+			if (graph.data('simpleTriggersHintbox')) {
+				data.isTriggerHintBoxFrozen = true;
+			}
+			else {
+				data.isHintBoxFrozen = true;
+			}
+
 			graph.data('widget')._pauseUpdating();
 
 			Overlay.prototype.recoverFocus.call({'$dialogue': graph.hintBoxItem});
@@ -126,6 +133,8 @@ jQuery(function ($) {
 
 			graph.hintBoxItem.on('onDeleteHint.hintBox', function(e) {
 				graph.data('widget')._resumeUpdating();
+
+				data.isTriggerHintBoxFrozen = false;
 				data.isHintBoxFrozen = false; // Unfreeze because only onfrozen hintboxes can be removed.
 				graph.off('mouseup', hintboxSilentMode);
 				destroyHintbox(graph);
@@ -152,6 +161,11 @@ jQuery(function ($) {
 			showHintbox(e, graph);
 			makeHintboxStatic(e, graph);
 		}
+		if (data.isTriggerHintBoxFrozen) {
+			data.isTriggerHintBoxFrozen = false;
+			showSimpleTriggerHintbox(e, graph);
+			makeHintboxStatic(e, graph);
+		}
 	}
 
 	// Method to start selection of some horizontal area in graph.
@@ -164,7 +178,7 @@ jQuery(function ($) {
 
 		if (data.dimX <= offsetX && offsetX <= data.dimX + data.dimW && data.dimY <= e.offsetY
 				&& e.offsetY <= data.dimY + data.dimH) {
-			$(document)
+			jQuery(document)
 				.on('selectstart', disableSelect)
 				.on('keydown', {graph: graph}, sBoxKeyboardInteraction)
 				.on('mousemove', {graph: graph}, moveSBoxMouse)
@@ -180,8 +194,8 @@ jQuery(function ($) {
 
 		var graph = e.data.graph,
 			data = graph.data('options'),
-			sbox = $('.svg-graph-selection', graph),
-			stxt = $('.svg-graph-selection-text', graph),
+			$sbox = jQuery('.svg-graph-selection', graph),
+			$stxt = jQuery('.svg-graph-selection-text', graph),
 			offsetX = e.clientX - graph.offset().left;
 
 		data.end = offsetX - data.dimX;
@@ -190,6 +204,7 @@ jQuery(function ($) {
 		if (data.start != data.end && !data.boxing) {
 			graph.data('widget')._pauseUpdating();
 			data.isHintBoxFrozen = false;
+			data.isTriggerHintBoxFrozen = false;
 			data.boxing = true;
 			destroyHintbox(graph);
 			hideHelper(graph);
@@ -199,7 +214,7 @@ jQuery(function ($) {
 			data.end = Math.min(offsetX - data.dimX, data.dimW);
 			data.end = (data.end > 0) ? data.end : 0;
 
-			sbox.attr({
+			$sbox.attr({
 				'x': (Math.min(data.start, data.end) + data.dimX) + 'px',
 				'y': data.dimY + 'px',
 				'width': Math.abs(data.end - data.start) + 'px',
@@ -210,7 +225,7 @@ jQuery(function ($) {
 				label = formatTimestamp(seconds, false, true)
 					+ (seconds < data.minPeriod ? ' [min 1' + t('S_MINUTE_SHORT') + ']'  : '');
 
-			stxt
+			$stxt
 				.text(label)
 				.attr({
 					'x': (Math.min(data.start, data.end) + data.dimX + 5) + 'px',
@@ -238,7 +253,7 @@ jQuery(function ($) {
 				to_offset = Math.floor(data.dimW - Math.max(data.start, data.end)) * data.spp;
 
 			if (seconds > data.minPeriod && (from_offset > 0 || to_offset > 0)) {
-				$.publish('timeselector.rangeoffset', {
+				jQuery.publish('timeselector.rangeoffset', {
 					from_offset: Math.max(0, Math.ceil(from_offset)),
 					to_offset: Math.ceil(to_offset)
 				});
@@ -348,15 +363,18 @@ jQuery(function ($) {
 							? direction_string.substr(1).replace(/([ML])\s(\d+)\s(\d+)/g, '$1$2\,$3').split(' ')
 							: direction_string.substr(1).split(' '),
 						index = direction.length,
-						point;
+						point,
+						point_label;
 
 					while (index) {
 						index--;
 						point = direction[index].substr(1).split(',');
-						if (x >= parseInt(point[0])) {
+						point_label = label[data_set === 'line' ? index : Math.ceil(index / 2)];
+
+						if (x >= parseInt(point[0]) && point_label !== '') {
 							px = point[0];
 							py = point[1];
-							pv = label[data_set === 'line' ? index : Math.ceil(index / 2)];
+							pv = point_label;
 							break;
 						}
 					}
@@ -419,7 +437,7 @@ jQuery(function ($) {
 	function repositionHintBox(e, graph) {
 		// Use closest positioned ancestor for offset calculation.
 		var offset = graph.parent().offsetParent().offset(),
-			hbox = $(graph.hintBoxItem),
+			hbox = jQuery(graph.hintBoxItem),
 			page_bottom = jQuery(window.top).scrollTop() + jQuery(window.top).height(),
 			mouse_distance = 15,
 			l = (document.body.clientWidth >= e.clientX + hbox.outerWidth() + mouse_distance)
@@ -445,155 +463,161 @@ jQuery(function ($) {
 			in_values_area = false,
 			in_problem_area = false;
 
-		if (data.boxing === false) {
-			// Check if mouse in the horizontal area in which hintbox must be shown.
-			in_x = (data.dimX <= offsetX && offsetX <= data.dimX + data.dimW);
-			in_problem_area = in_x && (data.dimY + data.dimH <= e.offsetY && e.offsetY <= data.dimY + data.dimH + 15);
-			in_values_area = in_x && (data.dimY <= e.offsetY && e.offsetY <= data.dimY + data.dimH);
+		if (graph.data('simpleTriggersHintbox') || data.isTriggerHintBoxFrozen === true) {
+			return;
+		}
 
-			// Show problems when mouse is in the 15px high area under the graph canvas.
-			if (data.showProblems && data.isHintBoxFrozen === false && in_problem_area) {
-				hideHelper(graph);
+		if (data.boxing === true) {
+			hideHelper(graph);
+			return;
+		}
 
-				var problems = findProblems(graph[0], e.offsetX),
-					problems_total = problems.length;
-				if (problems_total > 0) {
-					var tbody = $('<tbody>');
+		// Check if mouse in the horizontal area in which hintbox must be shown.
+		in_x = (data.dimX <= offsetX && offsetX <= data.dimX + data.dimW);
+		in_problem_area = in_x && (data.dimY + data.dimH <= e.offsetY && e.offsetY <= data.dimY + data.dimH + 15);
+		in_values_area = in_x && (data.dimY <= e.offsetY && e.offsetY <= data.dimY + data.dimH);
 
-					problems.slice(0, data.hintMaxRows).forEach(function(val, i) {
-						tbody.append(
-							$('<tr>')
-								.append($('<td>').append($('<a>', {'href': val.url}).text(val.clock)))
-								.append($('<td>').append(val.r_eventid
-									? $('<a>', {'href': val.url}).text(val.r_clock)
-									: val.r_clock)
-								)
-								.append($('<td>').append($('<span>', {'class': val.status_color}).text(val.status)))
-								.append($('<td>', {'class': val.severity}).text(val.name))
-						);
-					});
+		// Show problems when mouse is in the 15px high area under the graph canvas.
+		if (data.showProblems && data.isHintBoxFrozen === false && in_problem_area) {
+			hideHelper(graph);
 
-					html = $('<div>')
-							.addClass('svg-graph-hintbox')
-							.append(
-								$('<table>')
-									.addClass('list-table compact-view')
-									.append(tbody)
+			var problems = findProblems(graph[0], e.offsetX),
+				problems_total = problems.length;
+			if (problems_total > 0) {
+				var tbody = jQuery('<tbody>');
+
+				problems.slice(0, data.hintMaxRows).forEach(function(val, i) {
+					tbody.append(
+						jQuery('<tr>')
+							.append(jQuery('<td>').append(jQuery('<a>', {'href': val.url}).text(val.clock)))
+							.append(jQuery('<td>').append(val.r_eventid
+								? jQuery('<a>', {'href': val.url}).text(val.r_clock)
+								: val.r_clock)
 							)
-							.append(problems_total > data.hintMaxRows
-								? makeHintBoxFooter(data.hintMaxRows, problems_total)
-								: null
-							);
-				}
-			}
-			// Show graph values if mouse is over the graph canvas.
-			else if (in_values_area) {
-				// Set position of mouse following helper line.
-				setHelperPosition(e, graph);
-
-				// Find values.
-				var points = findValues(graph[0], offsetX),
-					points_total = points.length,
-					show_hint = false,
-					xy_point = false,
-					tolerance;
-
-				/**
-				 * Decide if one specific value or list of all matching Xs should be highlighted and either to show or
-				 * hide hintbox.
-				 */
-				if (data.isHintBoxFrozen === false) {
-					points.forEach(function(point) {
-						if (!show_hint && point.v !== null) {
-							show_hint = true;
-						}
-
-						tolerance = getDataPointTolerance(point.g);
-						if (!xy_point && point.v !== null
-								&& (+point.x + tolerance) > e.offsetX && e.offsetX > (+point.x - tolerance)
-								&& (+point.y + tolerance) > e.offsetY && e.offsetY > (+point.y - tolerance)) {
-							xy_point = point;
-							points_total = 1;
-						}
-					});
-				}
-
-				// Make html for hintbox.
-				if (show_hint) {
-					html = $('<ul>');
-				}
-				var rows_added = 0;
-				points.forEach(function(point) {
-					var point_highlight = point.g.querySelectorAll('.svg-point-highlight')[0];
-
-					if (point.v !== null && (xy_point === false || xy_point === point)) {
-						point_highlight.setAttribute('cx', point.x);
-						point_highlight.setAttribute('cy', point.y);
-
-						if (point.p > 0) {
-							point_highlight.setAttribute('cx', parseInt(point.x) + parseInt(point.p));
-						}
-
-						if (show_hint && data.hintMaxRows > rows_added) {
-							$('<li>')
-								.text(point.g.getAttribute('data-metric') + ': ' + point.v)
-								.append(
-									$('<span>')
-										.css('background-color', point.g.getAttribute('data-color'))
-										.addClass('svg-graph-hintbox-item-color')
-								)
-								.appendTo(html);
-							rows_added++;
-						}
-					}
-					else {
-						point_highlight.setAttribute('cx', -10);
-						point_highlight.setAttribute('cy', -10);
-					}
+							.append(jQuery('<td>').append(
+								jQuery('<span>', { 'class': val.status_color }).text(val.status))
+							)
+							.append(jQuery('<td>', {'class': val.severity}).text(val.name))
+					);
 				});
 
-				if (show_hint) {
-					// Calculate time at mouse position.
-					var time = parseInt(data.timeFrom) + parseInt((offsetX - data.dimX) * data.spp);
+				html = jQuery('<div>')
+						.addClass('svg-graph-hintbox')
+						.append(
+							jQuery('<table>')
+								.addClass('list-table compact-view')
+								.append(tbody)
+						)
+						.append(problems_total > data.hintMaxRows
+							? makeHintBoxFooter(data.hintMaxRows, problems_total)
+							: null
+						);
+			}
+		}
+		// Show graph values if mouse is over the graph canvas.
+		else if (in_values_area) {
+			// Set position of mouse following helper line.
+			setHelperPosition(e, graph);
 
-					html = $('<div>')
-							.addClass('svg-graph-hintbox')
+			// Find values.
+			var points = findValues(graph[0], offsetX),
+				points_total = points.length,
+				show_hint = false,
+				xy_point = false,
+				tolerance;
+
+			/**
+			 * Decide if one specific value or list of all matching Xs should be highlighted and either to show or
+			 * hide hintbox.
+			 */
+			if (data.isHintBoxFrozen === false) {
+				points.forEach(function(point) {
+					if (!show_hint && point.v !== null) {
+						show_hint = true;
+					}
+
+					tolerance = getDataPointTolerance(point.g);
+					if (!xy_point && point.v !== null
+							&& (+point.x + tolerance) > e.offsetX && e.offsetX > (+point.x - tolerance)
+							&& (+point.y + tolerance) > e.offsetY && e.offsetY > (+point.y - tolerance)) {
+						xy_point = point;
+						points_total = 1;
+					}
+				});
+			}
+
+			// Make html for hintbox.
+			if (show_hint) {
+				html = jQuery('<ul>');
+			}
+			var rows_added = 0;
+			points.forEach(function(point) {
+				var point_highlight = point.g.querySelectorAll('.svg-point-highlight')[0];
+
+				if (point.v !== null && (xy_point === false || xy_point === point)) {
+					point_highlight.setAttribute('cx', point.x);
+					point_highlight.setAttribute('cy', point.y);
+
+					if (point.p > 0) {
+						point_highlight.setAttribute('cx', parseInt(point.x) + parseInt(point.p));
+					}
+
+					if (show_hint && data.hintMaxRows > rows_added) {
+						jQuery('<li>')
+							.text(point.g.getAttribute('data-metric') + ': ' + point.v)
 							.append(
-								$('<div>')
-									.addClass('header')
-									.html(time2str(time))
+								jQuery('<span>')
+									.css('background-color', point.g.getAttribute('data-color'))
+									.addClass('svg-graph-hintbox-item-color')
 							)
-							.append(html)
-							.append(points_total > data.hintMaxRows
-								? makeHintBoxFooter(data.hintMaxRows, points_total)
-								: null
-							);
-				}
-			}
-			else {
-				hideHelper(graph);
-			}
-
-			if (html !== null) {
-				if (hbox === null) {
-					hbox = hintBox.createBox(e, graph, html, '', false, false, graph.parent());
-					graph
-						.off('mouseup', makeHintboxStatic)
-						.on('mouseup', {graph: graph}, makeHintboxStatic);
-					graph.data('hintbox', hbox);
+							.appendTo(html);
+						rows_added++;
+					}
 				}
 				else {
-					hbox.find('> div').replaceWith(html);
+					point_highlight.setAttribute('cx', -10);
+					point_highlight.setAttribute('cy', -10);
 				}
+			});
 
-				graph.hintBoxItem = hbox;
-				repositionHintBox(e, graph);
+			if (show_hint) {
+				// Calculate time at mouse position.
+				var time = parseInt(data.timeFrom) + parseInt((offsetX - data.dimX) * data.spp);
+
+				html = jQuery('<div>')
+						.addClass('svg-graph-hintbox')
+						.append(
+							jQuery('<div>')
+								.addClass('header')
+								.html(time2str(time))
+						)
+						.append(html)
+						.append(points_total > data.hintMaxRows
+							? makeHintBoxFooter(data.hintMaxRows, points_total)
+							: null
+						);
 			}
 		}
 		else {
 			hideHelper(graph);
 		}
 
+		if (html !== null) {
+			if (hbox === null) {
+				hbox = hintBox.createBox(e, graph, html, '', false, false, graph.parent());
+				graph
+					.off('mouseup', makeHintboxStatic)
+					.on('mouseup', {graph: graph}, makeHintboxStatic);
+				graph.data('hintbox', hbox);
+			}
+			else {
+				hbox.find('> div').replaceWith(html);
+			}
+
+			graph.hintBoxItem = hbox;
+			repositionHintBox(e, graph);
+		}
+
 		if (html === null && (in_values_area || in_problem_area)) {
 			destroyHintbox(graph);
 		}
@@ -601,13 +625,13 @@ jQuery(function ($) {
 
 	// Function creates hintbox footer.
 	function makeHintBoxFooter(num_displayed, num_total) {
-		return $('<div>')
+		return jQuery('<div>')
 			.addClass('table-paging')
 			.append(
-				$('<div>')
+				jQuery('<div>')
 					.addClass('paging-btn-container')
 					.append(
-						$('<div>')
+						jQuery('<div>')
 							.text(sprintf(t('S_DISPLAYING_FOUND'), num_displayed, num_total))
 							.addClass('table-stats')
 					)
@@ -624,8 +648,10 @@ jQuery(function ($) {
 						dimW: widget._svg_options.dims.w,
 						dimH: widget._svg_options.dims.h,
 						showProblems: widget._svg_options.show_problems,
+						showSimpleTriggers: widget._svg_options.show_simple_triggers,
 						hintMaxRows: widget._svg_options.hint_max_rows,
 						isHintBoxFrozen: false,
+						isTriggerHintBoxFrozen: false,
 						spp: widget._svg_options.spp || null,
 						timeFrom: widget._svg_options.time_from,
 						minPeriod: widget._svg_options.min_period,
@@ -641,11 +667,14 @@ jQuery(function ($) {
 			});
 		},
 		activate: function () {
-			const widget = $(this).data('widget');
+			const widget = jQuery(this).data('widget');
 			const graph = jQuery(widget._svg);
 
 			graph
-				.on('mousemove', {graph}, showHintbox)
+				.on('mousemove', (e) => {
+					showSimpleTriggerHintbox(e, graph);
+					showHintbox(e, graph);
+				})
 				.on('mouseleave', function() {
 					destroyHintbox(graph);
 					hideHelper(graph);
@@ -656,14 +685,14 @@ jQuery(function ($) {
 				graph
 					.on('dblclick', function() {
 						hintBox.hideHint(graph, true);
-						$.publish('timeselector.zoomout');
+						jQuery.publish('timeselector.zoomout');
 						return false;
 					})
 					.on('mousedown', {graph}, startSBoxDrag);
 			}
 		},
 		deactivate: function (e) {
-			const widget = $(this).data('widget');
+			const widget = jQuery(this).data('widget');
 			const graph = jQuery(widget._svg);
 
 			destroySBox(e, graph);
@@ -671,7 +700,122 @@ jQuery(function ($) {
 		}
 	};
 
-	$.fn.svggraph = function(method) {
+	function showSimpleTriggerHintbox(e, graph) {
+		graph = graph || e.data.graph;
+		const data = graph.data('options');
+		let hbox = graph.data('hintbox') || null;
+
+		graph.data('simpleTriggersHintbox', false);
+		let html = null;
+
+		if (!data.showSimpleTriggers || data.isHintBoxFrozen === true) {
+			return;
+		}
+
+		if (data.boxing === true) {
+			hideHelper(graph);
+			return;
+		}
+
+		// Check if mouse in the horizontal area in which hintbox must be shown.
+		const triggers = findTriggers(graph[0], e.offsetY);
+		if (data.isTriggerHintBoxFrozen === false && triggers) {
+			hideHelper(graph);
+
+			const triggers_length = triggers.length;
+			if (triggers_length > 0) {
+				const hint_body = jQuery('<ul></ul>');
+
+				const trigger_areas = triggers.filter((val) => {
+					if (val.begin_position > e.offsetX) {
+						return false;
+					}
+
+					if (e.offsetX > val.end_position) {
+						return false;
+					}
+
+					return true;
+				});
+
+				if (!trigger_areas.length) {
+					return;
+				}
+
+				triggers.slice(0, data.hintMaxRows).forEach((val, i) => {
+					hint_body.append(
+						jQuery('<li>')
+							.text(val.trigger + ' [' + val.constant + ']')
+							.append(
+								jQuery('<span>')
+									.css('background-color', val.color)
+									.addClass('svg-graph-hintbox-trigger-color')
+							)
+					)
+
+					val.elem.classList.toggle('svg-graph-simple-trigger-hover', true)
+				});
+
+				html = jQuery('<div>')
+					.addClass('svg-graph-hintbox')
+					.append(hint_body)
+					.append(triggers_length > data.hintMaxRows
+						? makeHintBoxFooter(data.hintMaxRows, triggers_length)
+						:null
+					);
+
+				graph.data('simpleTriggersHintbox', true);
+			}
+		}
+
+		if (html !== null) {
+			if (hbox === null) {
+				hbox = hintBox.createBox(e, graph, html, '', false, false, graph.parent());
+				graph
+					.off('mouseup', makeHintboxStatic)
+					.on('mouseup', {graph: graph}, makeHintboxStatic);
+				graph.data('hintbox', hbox);
+			}
+			else {
+				hbox.find('> div').replaceWith(html);
+			}
+
+			graph.hintBoxItem = hbox;
+			repositionHintBox(e, graph);
+		}
+		else {
+			destroyHintbox(graph);
+		}
+	}
+
+	function findTriggers(graph, y) {
+		const triggers = [];
+		const nodes = graph.querySelectorAll('.svg-graph-simple-trigger line');
+
+		[...graph.querySelectorAll('.svg-graph-simple-trigger.svg-graph-simple-trigger-hover')].map(
+			(elem) => elem.classList.toggle('svg-graph-simple-trigger-hover', false)
+		);
+
+
+		for (var i = 0, l = nodes.length; l > i; i++) {
+			const trigger_y = parseInt(nodes[i].getAttribute('y1'));
+
+			if (y < trigger_y + 10 && y > trigger_y - 10) {
+				triggers.push({
+					begin_position: parseInt(nodes[i].getAttribute('x1')),
+					end_position: parseInt(nodes[i].getAttribute('x2')),
+					color: nodes[i].parentElement.getAttribute('severity-color'),
+					constant: nodes[i].parentElement.getAttribute('constant'),
+					trigger: nodes[i].parentElement.getAttribute('description'),
+					elem: nodes[i].parentElement
+				});
+			}
+		}
+
+		return triggers;
+	}
+
+	jQuery.fn.svggraph = function(method) {
 		if (methods[method]) {
 			return methods[method].apply(this, Array.prototype.slice.call(arguments, 1));
 		}
